<?php

namespace App\Http\Livewire\Admin;

use App\Models\AttributeValue;
use App\Models\Category;
use App\Models\Product;
use App\Models\ProductAttribute;
use App\Models\Subcategory;
use Carbon\Carbon;
use Livewire\Component;
use Illuminate\Support\Str;
use Livewire\WithFileUploads;
use Illuminate\Support\Facades\DB;

class AdminAddProductComponent extends Component
{
    use WithFileUploads;

    public $name;
    public $slug;
    public $short_description;
    public $description;
    public $regular_price;
    public $sale_price = 0;
    public $SKU;
    public $stock_status;
    public $featured;
    public $quantity;
    public $image;
    public $images;
    public $category_id;
    public $subcategory_id;
    public $attr;
    // public $inputs = [];
    public $attr_arr = [];
    public $attr_values;

    protected $messages = [
        'SKU.required' => 'Enter an SKU for this item'
    ];

    public function mount()
    {
        $this->stock_status = 'in-stock';
        $this->featured = 0;
    }

    public function add()
    {       
        if(!in_array($this->attr, $this->attr_arr)){
            // array_push($this->inputs, $this->attr);
            array_push($this->attr_arr, $this->attr);
        }
    }

     public function remove($attr)
    {
        unset($this->attr_arr[$attr]);
    }

    public function generateSlug()
    {
        $this->slug = Str::slug($this->name);
    }

    public function updated($fields)
    {
        $this->validateOnly($fields, [
            "name" => 'required',
            "slug" => 'required|unique:products',
            "short_description" => 'required',
            "description" => 'required',
            "regular_price" => 'required|numeric',
            "sale_price" => 'sometimes|numeric',
            "SKU" => 'required',
            "stock_status" => 'required',
            "featured" => 'required|boolean',
            "quantity" => 'required|numeric',
            "image" => 'required|mimes:jpeg,jpg,png',
            "category_id" => 'required', 
        ]);
    }

    public function storeProduct()
    {
        $this->validate([
            "name" => 'required',
            "slug" => 'required|unique:products',
            "short_description" => 'required',
            "description" => 'required',
            "regular_price" => 'required|numeric',
            "sale_price" => 'nullable|numeric',
            "SKU" => 'required|unique:products',
            "stock_status" => 'required',
            "featured" => 'required|boolean',
            "quantity" => 'required|numeric',
            "image" => 'required|mimes:jpeg,jpg,png',
            "category_id" => 'required', 
        ]);
        
        $imageName = Carbon::now()->timestamp . '.' . $this->image->extension();
        $this->image->storeAs('products', $imageName);

        if($this->images){
            $imagesName = '';
            foreach($this->images as $key=>$image){
                $imgName = Carbon::now()->timestamp . $key . '.' . $this->image->extension();
                $image->storeAs('products', $imgName);
                $imagesName = $imagesName . ',' . $imgName;
            }
        }
        
        DB::beginTransaction();
        try{
            $product = Product::create([
                'name' => $this->name,
                'slug' => $this->slug,
                'short_description' => $this->short_description,
                'description' => $this->description,
                'regular_price' => $this->regular_price,
                'sale_price' => $this->sale_price,
                'SKU' => $this->SKU,
                'stock_status' => $this->stock_status,
                'featured' => $this->featured,
                'quantity' => $this->quantity,
                'image' => $imageName,
                'images' => $imagesName ? $imagesName : null,
                'category_id' => $this->category_id,
                'subcategory_id' => $this->subcategory_id ? $this->subcategory_id : null,
            ]);

            foreach($this->attr_values as $key => $attr_value)
            {
                $values = explode(",", $attr_value);
                foreach($values as $value){
                    $attrVal = new AttributeValue();
                    $attrVal->attribute_id = $key;
                    $attrVal->value = $value;
                    $attrVal->product_id = $product->id;
                    $attrVal->save();
                }
            }
            DB::commit();

            $this->reset();

            session()->flash('message', 'Product has been created successfully');
        }catch(\Throwable $err){
            DB::rollBack();
            
            return redirect()->back()->with('error', "Post creation failed. {$err->getMessage()}");
        }


    }

    public function changeSubcategory(){
        $this->subcategory_id = 0;
    }

    public function render()
    {
        $statuses = [
                'in-stock' => 'In Stock' ,
                'out-of-stock' => 'Out of Stock'
        ];

        $categories = Category::all();
        $subcategories = Subcategory::where('category_id', $this->category_id)->get();
        $attributes = ProductAttribute::all();
        return view('livewire.admin.admin-add-product-component', compact('categories', 'subcategories','statuses', 'attributes'));
    }
}
